

import requests
from .conf import domain_dict, token_url_dict, host_url_dict


class Auth:

    def __init__(self, client_id, client_secret, redirect_uri, region):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.region = region
        self.domain = domain_dict[self.region]
        self.token_url = token_url_dict[self.region]
        self.host_url = host_url_dict[self.region]
        self.scope = "cpc_advertising:campaign_management"
        self.response_type = "code"
        self.headers = {
            "Content_Type": "application/x-www-form-urlencoded:charset=UTF-8"
        }
        self.uri_path = ""

    def get_grant_url(self):
        return "{}?client_id={}&scope={}&response_type={}&redirect_uri={}".format(self.host_url,
                                                                                  self.client_id,
                                                                                  self.scope,
                                                                                  self.response_type,
                                                                                  self.redirect_uri)

    def get_refresh_token(self, code):
        data = {
            "grant_type": "authorization_code",
            "code": code,
            "redirect_uri": self.redirect_uri,
            "client_id": self.client_id,
            "client_secret": self.client_secret
        }
        response = requests.post(self.token_url, data, headers=self.headers)
        return response

    def get_new_access_token(self, refresh_token):
        data = {
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
            "client_id": self.client_id,
            "client_secret": self.client_secret
        }
        response = requests.post(self.token_url, data, headers=self.headers)
        return response




from ..client import Client


class NegativeTargeting(Client):

    def get_negative_targets(self, start_index: int = 0, count: int = None,
                             state_filter: str = "enabled, paused, archived",
                             ad_group_id_filter: str = None, campaign_id_filter: str = None):
        self.uri_path = "/sd/negativeTargets"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "campaignIdFilter": campaign_id_filter
        }
        return self.execute()

    def update_neg_targets(self, data):
        self.uri_path = "/sd/negativeTargets"
        self.method = "put"
        self.data = data
        return self.execute()

    def create_net_targets(self, data):
        self.uri_path = "/sd/negativeTargets"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_neg_targets_by_id(self, target_id):
        self.uri_path = "/sd/negativeTargets/{}".format(target_id)
        self.method = "get"
        return self.execute()

    def delete_neg_targets_by_id(self, target_id):
        self.uri_path = "/sd/negativeTargets/{}".format(target_id)
        self.method = "delete"
        return self.execute()

    def get_neg_targets_extended(self, start_index: int = 0, count: int = None,
                                 state_filter: str = "enabled, paused, archived",
                                 ad_group_id_filter: str = None, campaign_id_filter: str = None):
        self.uri_path = "/sd/negativeTargets/extended"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "campaignIdFilter": campaign_id_filter
        }
        return self.execute()

    def get_neg_targets_extended_by_id(self, target_id):
        self.uri_path = "/sd/negativeTargets/extended/{}".format(target_id)
        self.method = "get"
        return self.execute()





from ..client import Client


class Stores(Client):

    def get_store_assets(self, brand_entity_id: str = None, media_type: str = None):
        self.method = "get"
        self.uri_path = "/stores/assets"
        self.params = {
            "brandEntityId": brand_entity_id,
            "mediaType": media_type
        }
        return self.execute()

    def create_store_assets(self, content_disposition: str, content_type: str, data=None):
        self.uri_path = "/store/assets"
        self.headers["Content-Type"] = content_type
        self.headers["Content-Disposition"] = content_disposition
        self.method = "post"
        self.data = data
        return self.execute()




from ..client import Client


class BudgetRulesRecommendation(Client):

    def get_campaigns_budget_rules_recommendations(self, data):
        self.uri_path = "/sp/campaigns/budgetRules/recommendations"
        self.method = "post"
        self.data = data
        return self.execute()




from ..client import Client


class BidRecommendations(Client):

    def get_bid_by_ad_group_id(self, ad_group_id):
        self.method = "get"
        self.uri_path = "/v2/sp/adGroups/{}/bidRecommendations".format(ad_group_id)
        return self.execute()

    def get_bid_by_keyword_id(self, keyword_id):
        self.method = "get"
        self.uri_path = "/v2/sp/keywords/{}/bidRecommendations".format(keyword_id)
        return self.execute()

    def get_bid_by_keywords(self, data):
        self.uri_path = "/v2/sp/keywords/bidRecommendations"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_bid_by_targets(self, data):
        self.uri_path = "/v2/sp/targets/bidRecommendations"
        self.method = "post"
        self.data = data
        return self.execute()


class KeywordRecommendations(Client):

    def get_keywords_recommendations(self, data):
        self.method = "post"
        self.uri_path = "/sp/targets/keywords/recommendations"
        self.data = data
        return self.execute()




from ..client import Client


class BrandSafetyList(Client):

    def get_brand_safety_deny(self, start_index: int = 0, count: int = 1000):
        self.uri_path = "/sd/brandSafety/deny"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count
        }
        return self.execute()

    def create_brand_safety_deny(self, data):
        self.uri_path = "/sd/brandSafety/deny"
        self.method = "post"
        self.data = data
        return self.execute()

    def delete_brand_safety_deny(self):
        self.uri_path = "/sd/brandSafety/deny"
        self.method = "delete"
        return self.execute()

    def get_brand_safety_results_by_id(self, request_id: str, start_index: int = 0, count: int = 1000):
        self.uri_path = "/sd/brandSafety/{}/results".format(request_id)
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count
        }
        return self.execute()

    def get_brand_safety_status_by_id(self, request_id):
        self.uri_path = "/sd/brandSafety/{}/status".format(request_id)
        self.method = "get"
        return self.execute()

    def get_brand_safety_statuses(self):
        self.uri_path = "/sd/brandSafety/status"
        self.method = "get"
        return self.execute()





from ..client import Client


class BudgetRecommendations(Client):

    def get_campaigns_budget_recommendations(self, campaign_ids: list):
        self.uri_path = "/sp/campaigns/budgetRecommendations"
        self.method = "post"
        self.data = {
            "campaignIds": campaign_ids
        }
        return self.execute()





from ..client import Client


class ProductTargeting(Client):

    def get_targets(self, next_token: str = None, max_results: int = 0, filters: list = None):
        self.uri_path = "/sb/targets/list"
        self.method = "post"

        self.data = {
            "nextToken": next_token,
            "maxResults": max_results,
            "filters": filters
        }
        return self.execute()

    def update_targets(self, data):
        self.uri_path = "/sb/targets"
        self.method = "put"
        self.data = data
        return self.execute()

    def create_targets(self, data):
        self.uri_path = "/sb/targets"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_targets_by_id(self, target_id):
        self.uri_path = "/sb/targets/{}".format(target_id)
        self.method = "get"
        return self.execute()

    def delete_targets_by_id(self, target_id):
        self.uri_path = "/sb/targets/{}".format(target_id)
        self.method = "delete"
        return self.execute()






from ..client import Client


class TargetingRecommendations(Client):

    def get_targets_recommendations(self, tactic: str, products: list, type_filter: list):
        self.uri_path = "/sd/targets/recommendations"
        self.method = "post"
        self.data = {
            "tactic": tactic,
            "products": products,
            "typeFilter": type_filter
        }
        return self.execute()



from ..client import Client


class BidRecommendations(Client):

    def get_bid_recommendations(self, data):
        """
        Get a list of bid recommendation objects for a specified list of keywords or products.
        :param data: A list of keywords or targeting expressions for which to generate bid recommendations.
        Note that if a value is specified for the campaignId field, the past performance data for the
        campaign may be use to create bid recommendations.
        ExampleValue: {
        "campaignId": 0,
        "targets": [
        [
        {
        "type": "asinCategorySameAs",
        "value": "string"
        }
        ]
        ],
        "keywords": [
        {
        "matchType": "broad",
        "keywordText": "string"
        }
        ],
        "adFormat": "productCollection"
        }
        :return:
        """
        self.uri_path = "/sb/recommendations/bids"
        self.method = "post"
        self.data = data
        return self.execute()





from ..client import Client


class KeywordRecommendations(Client):

    def get_keyword_recommendations(self, params):
        self.uri_path = "/sb/recommendations/keyword"
        self.data = params
        self.method = "post"
        return self.execute()





domain_dict = {
    "NA": "https://advertising-api.amazon.com",
    "EU": "https://advertising-api-eu.amazon.com",
    "FE": "https://advertising-api-fe.amazon.com"
}
token_url_dict = {
    "NA": "https://api.amazon.com/auth/o2/token",
    "EU": "https://api.amazon.co.uk/auth/o2/token",
    "FE": "https://api.amazon.co.jp/auth/o2/token"
}
host_url_dict = {
    "NA": "https://www.amazon.com/ap/oa",
    "EU": "https://eu.account.amazon.com/ap/oa",
    "FE": "	https://apac.account.amazon.com/ap/oa"
}







from ..client import Client


class Snapshots(Client):

    def create_snapshot(self, record_type: str, state_filter=None):
        if state_filter is None:
            state_filter = {}
        self.uri_path = "/v2/hsa/{}/snapshot".format(record_type)
        self.method = "post"
        self.data = {
            "stateFilter": state_filter
        }
        return self.execute()

    def get_snapshots(self, snapshot_id: str):
        self.uri_path = "/v2/hsa/snapshots/{}".format(snapshot_id)
        self.method = "get"
        return self.execute()



from .adgroups import AdGroups
from .bid_recommendations import BidRecommendations
from .brands import Brands
from .campaigns import Campaigns
from .drafts import Drafts
from .keyword_recommendations import KeywordRecommendations
from .keywords import Keywords
from .landing_page_asin import LandingPageAsin
from .media import Media
from .moderation import Moderation
from .negative_keywords import NegativeKeywords
from .negative_product_targeting import NegativeProductTargeting
from .product_targeting import ProductTargeting
from .reports import Reports
from .snapshots import Snapshots
from .stores import Stores
from .targeting_recommendations import TargetingRecommendations










from ..client import Client


class Drafts(Client):

    def get_drafts_campaign(self, start_index: int = 0, count: int = None, name: str = None,
                            draft_campaign_id_filter: str = None, portfolio_id_filter: str = None,
                            ad_format_filter: str = None):
        self.method = "get"
        self.uri_path = "/sb/drafts/campaigns"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "name": name,
            "draftCampaignIdFilter": draft_campaign_id_filter,
            "portfolioIdFilter": portfolio_id_filter,
            "adFormatFilter": ad_format_filter
        }
        return self.execute()

    def create_drafts_campaign(self, data):
        self.uri_path = "/sb/drafts/campaigns"
        self.method = "post"
        self.data = data
        return self.execute()

    def update_drafts_campaign(self, data):
        self.uri_path = "/sb/drafts/campaigns"
        self.method = "put"
        self.data = data
        return self.execute()

    def get_drafts_campaign_by_id(self, draft_campaign_id):
        self.method = "get"
        self.uri_path = "/sb/drafts/campaigns/{}".format(draft_campaign_id)
        return self.execute()

    def delete_drafts_campaign_by_id(self, draft_campaign_id):
        self.uri_path = "/sb/drafts/campaigns/{}".format(draft_campaign_id)
        self.method = "delete"
        return self.execute()

    def submit_drafts_campaign(self, data):
        self.uri_path = "/sb/drafts/campaigns/submit"
        self.method = "post"
        self.data = data
        return self.execute()





from ..client import Client


class TargetingRecommendations(Client):

    def get_recommendations_targets(self, next_token: str = None, max_results: int = None, filters: list = None):
        self.uri_path = "/sb/recommendations/targets/product/list"
        self.method = "post"
        self.data = {
            "nextToken": next_token,
            "maxResults": max_results,
            "filters": filters
        }
        return self.execute()

    def get_recommendations_targets_category(self, asins):
        self.uri_path = "/sb/recommendations/targets/category"
        self.method = "post"
        self.data = {
            "asins": asins
        }
        return self.execute()

    def get_recommendations_targets_brand(self, category_id: int = None, keyword: str = None):
        self.uri_path = "/sb/recommendations/targets/brand"
        self.method = "post"
        self.data = {
            "categoryId": category_id,
            "keyword": keyword
        }
        return self.execute()






from ..client import Client


class SuggestProducts(Client):

    def get_suggest_products(self, params):
        self.method = "get"
        self.uri_path = "/sd/suggestedProducts"
        self.data = params
        return self.execute()

    def get_suggest_products_by_status(self, params):
        self.uri_path = "/sd/suggestedProducts/productReadinessStatus"
        self.method = "post"
        self.data = params
        return self.execute()





from ..client import Client


class Targeting(Client):

    def get_targets(self, start_index: int = 0, count: int = None, state_filter: str = "enabled, paused, archived",
                    ad_group_id_filter: str = None, campaign_id_filter: str = None):
        self.uri_path = "/sd/targets"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "campaignIdFilter": campaign_id_filter
        }
        return self.execute()

    def update_targets(self, data):
        self.uri_path = "/sd/targets"
        self.method = "put"
        self.data = data
        return self.execute()

    def create_targets(self, data):
        self.uri_path = "/sd/targets"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_targets_by_id(self, target_id):
        self.uri_path = "/sd/targets/{}".format(target_id)
        self.method = "get"
        return self.execute()

    def delete_targets_by_id(self, target_id):
        self.uri_path = "/sd/targets/{}".format(target_id)
        self.method = "delete"
        return self.execute()

    def get_targets_extended(self, start_index: int = 0, count: int = None,
                             state_filter: str = "enabled, paused, archived",
                             ad_group_id_filter: str = None, campaign_id_filter: str = None):
        self.uri_path = "/sd/targets/extended"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "campaignIdFilter": campaign_id_filter
        }
        return self.execute()

    def get_targets_extended_by_id(self, target_id):
        self.uri_path = "/sd/targets/extended/{}".format(target_id)
        self.method = "get"
        return self.execute()



from ..client import Client


class Creatives(Client):

    def get_creatives(self, start_index: int = 0, count: int = 100, ad_group_id_filter: str = None,
                      creative_id_filter: str = None):
        self.uri_path = "/sd/creatives"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "adGroupIdFilter": ad_group_id_filter,
            "creativeIdFilter": creative_id_filter
        }
        return self.execute()

    def update_creatives(self, data):
        self.uri_path = "/sd/creatives"
        self.method = "put"
        self.data = data
        return self.execute()

    def create_creatives(self, data):
        self.uri_path = "/sd/creatives"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_creative_preview(self, data):
        self.uri_path = "/sd/creatives/preview"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_creative_moderation(self, language: str = "en-US", start_index: int = 0, count: int = 100,
                                ad_group_id_filter: str = None, creative_id_filter: str = None):
        self.uri_path = "/sd/moderation/creatives"
        self.method = "get"
        self.params = {
            "language": language,
            "startIndex": start_index,
            "count": count,
            "adGroupIdFilter": ad_group_id_filter,
            "creativeIdFilter": creative_id_filter
        }
        return self.execute()




from ..client import Client


class NegativeProductTargeting(Client):

    def get_negative_targets(self, next_token: str = None, max_results: int = None, filters: list = None):
        self.uri_path = "/sb/negativeTargets/list"
        self.method = "post"
        self.data = {
            "nextToken": next_token,
            "maxResults": max_results,
            "filters": filters
        }
        return self.execute()

    def update_negative_targets(self, data):
        self.uri_path = "/sb/negativeTargets"
        self.method = "put"
        self.data = data
        return self.execute()

    def create_negative_targets(self, data):
        self.uri_path = "/sb/negativeTargets"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_negative_targets_by_id(self, negative_target_id):
        self.uri_path = "/sb/negativeTargets/{}".format(negative_target_id)
        self.method = "get"
        return self.execute()

    def delete_negative_targets_by_id(self, negative_target_id):
        self.uri_path = "/sb/negativeTargets/{}".format(negative_target_id)
        self.method = "delete"
        return self.execute()




from ..client import Client


class Keywords(Client):

    def get_keywords_by_id(self, keyword_id):
        self.method = "get"
        self.uri_path = "/v2/sp/keywords/{}".format(keyword_id)
        return self.execute()

    def delete_keywords_by_id(self, keyword_id):
        self.uri_path = "/v2/sp/keywords/{}".format(keyword_id)
        self.method = "delete"
        return self.execute()

    def get_keywords_extended(self, start_index: int = 0, count: int = None, match_type_filter: str = None,
                              keyword_text: str = None, state_filter: str = None, campaign_id_filter: str = None,
                              ad_group_id_filter: str = None, keyword_id_filter: str = None, locale: str = None):
        self.method = "get"
        self.uri_path = "/v2/sp/keywords/extended"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "matchTypeFilter": match_type_filter,
            "keywordText": keyword_text,
            "stateFilter": state_filter,
            "campaignIdFilter": campaign_id_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "keywordIdFilter": keyword_id_filter,
            "locale": locale
        }
        return self.execute()

    def get_keywords_extended_by_id(self, keyword_id):
        self.method = "get"
        self.uri_path = "/v2/sp/keywords/extended/{}".format(keyword_id)
        return self.execute()

    def get_keywords(self, start_index: int = 0, count: int = None, match_type_filter: str = None,
                     keyword_text: str = None, state_filter: str = None, campaign_id_filter: str = None,
                     ad_group_id_filter: str = None, keyword_id_filter: str = None, locale: str = None):
        self.method = "get"
        self.uri_path = "/v2/sp/keywords"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "matchTypeFilter": match_type_filter,
            "keywordText": keyword_text,
            "stateFilter": state_filter,
            "campaignIdFilter": campaign_id_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "keywordIdFilter": keyword_id_filter,
            "locale": locale
        }
        return self.execute()

    def create_keywords(self, data):
        self.uri_path = "/v2/sp/keywords"
        self.data = data
        self.method = "post"
        return self.execute()

    def update_keywords(self, data):
        self.uri_path = "/v2/sp/keywords"
        self.data = data
        self.method = "put"
        return self.execute()


class KeywordsV3(Client):

    def get_keywords_list(self, data):
        self.method = "post"
        self.uri_path = "/sp/keywords/list"
        self.data = data
        return self.execute()

    def update_keywords(self, data):
        self.method = "put"
        self.uri_path = "/sp/keywords"
        self.data = data
        return self.execute()

    def delete_keywords(self, data):
        self.method = "post"
        self.uri_path = "/sp/keywords/delete"
        self.data = data
        return self.execute()



from .adgroups import AdGroups, AdGroupsV3
from .bidrecommendations import BidRecommendations, KeywordRecommendations
from .budget_recommendations import BudgetRecommendations
from .budget_rules_recommendation import BudgetRulesRecommendation
from .campaigns import Campaigns, CampaignsV3
from .campaign_negative_keywords import CampaignNegativeKeywords
from .keywords import Keywords, KeywordsV3
from .negative_keywords import NegativeKeywords
from .negative_product_targeting import NegativeProductTargeting
from .product_ads import ProductAds, ProductAdsV3
from .product_targeting import ProductTargeting, ProductTargetingV3
from .reports import Reports, Reporting
from .snapshots import Snapshots
from .suggested_keywords import SuggestedKeywords





from ..client import Client


class Campaigns(Client):

    def create_campaigns(self, data):
        self.uri_path = "/sb/campaigns"
        self.method = "post"
        self.data = data
        return self.execute()

    def update_campaigns(self, data):
        self.uri_path = "/sb/campaigns"
        self.method = "put"
        self.data = data
        return self.execute()

    def get_campaigns(self, start_index: int = 0, count: int = None, state_filter: str = None,
                      name: str = None, portfolio_id_filter: str = None, campaign_id_filter: str = None,
                      ad_format_filter: str = None, creative_type: str = None):
        self.uri_path = "/sb/campaigns"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "name": name,
            "portfolioIdFilter": portfolio_id_filter,
            "campaignIdFilter": campaign_id_filter,
            "adFormatFilter": ad_format_filter,
            "creativeType": creative_type
        }
        return self.execute()

    def get_campaigns_by_id(self, campaign_id):
        self.uri_path = "/sb/campaigns/{}".format(campaign_id)
        self.method = "get"
        return self.execute()

    def delete_campaigns_by_id(self, campaign_id):
        self.uri_path = "/sb/campaigns/{}".format(campaign_id)
        self.method = "delete"
        return self.execute()




from ..client import ProClient


class Profiles(ProClient):

    def register(self, **params):
        self.uri_path = "/v2/profiles/register"
        self.data = params
        self.method = "put"
        return self.execute()

    def get_profiles(self):
        self.uri_path = "/v2/profiles"
        self.method = "get"
        return self.execute()




from ..client import Client


class Reports(Client):

    def request_report(self, record_type, data):
        self.uri_path = "/v2/sp/{}/report".format(record_type)
        self.method = "post"
        self.data = data
        return self.execute()

    def get_report(self, report_id):
        self.uri_path = "/v2/reports/{}".format(report_id)
        self.method = "get"
        return self.execute()

    def get_report_download_url(self, location):
        self.method = "get"
        return self.execute_download(location)

    def get_report_download(self, report_id):
        self.uri_path = "/v2/reports/{}/download".format(report_id)
        self.method = "get"
        self.headers.pop("Content-Type")
        self.headers["Accept-encoding"] = "gzip"
        return self.execute()




from ..client import Client


class ProductMetadata(Client):

    def get_product_metadata(self, data):
        self.uri_path = "/product/metadata"
        self.method = "post"
        self.data = data
        return self.execute()




from ..client import Client


class Reports(Client):

    def request_sb_report(self, record_type, report_date, metrics, segment=None, creative_type=None):
        self.uri_path = "/v2/hsa/{}/report".format(record_type)
        self.method = "post"
        self.data = {
            "segment": segment,
            "reportDate": report_date,
            "metrics": metrics,
            "creativeType": creative_type
        }
        return self.execute()

    def request_sd_report(self, record_type, report_date, metrics, tactic):
        self.uri_path = "/sd/{}/report".format(record_type)
        self.method = "post"
        self.data = {
            "reportDate": report_date,
            "tactic": tactic,
            "metrics": metrics
        }
        return self.execute()

    def request_sp_report(self, record_type, report_date, metrics, state_filter=None, campaign_type=None, segment=None):
        self.uri_path = "/v2/sp/{}/report".format(record_type)
        self.method = "post"
        self.data = {
            "stateFilter": state_filter,
            "campaignType": campaign_type,
            "segment": segment,
            "reportDate": report_date,
            "metrics": metrics
        }
        return self.execute()

    def get_report(self, report_id):
        self.uri_path = "/v2/reports/{}".format(report_id)
        self.method = "get"
        return self.execute()

    def get_report_download_url(self, location):
        self.method = "get"
        return self.execute_download(location)

    def get_report_download(self, report_id):
        self.uri_path = "/v2/reports/{}/download".format(report_id)
        self.method = "get"
        self.headers.pop("Content-Type")
        self.headers["Accept-encoding"] = "gzip"
        return self.execute()


class Reporting(Client):

    def create_reports(self, data):
        self.method = "post"
        self.uri_path = "/reporting/reports"
        self.data = data
        return self.execute()

    def get_reports(self, report_id):
        self.method = "get"
        self.uri_path = "/reporting/reports/{}".format(report_id)
        return self.execute()

    def delete_reports(self, report_id):
        self.method = "delete"
        self.uri_path = "/reporting/reports/{}".format(report_id)
        return self.execute()



from .adgroups import AdGroups
from .bid_recommendations import BidRecommendations
from .budget_rules import BudgetRules
from .campaigns import Campaigns
from .negative_targeting import NegativeTargeting
from .product_ads import ProductAds
from .reports import Reports
from .snapshots import Snapshots
from .suggest_products import SuggestProducts
from .targeting import Targeting
from .targeting_recommendations import TargetingRecommendations






from ..client import Client


class AdGroups(Client):

    def get_ad_groups(self, start_index: int = 0, count: int = None, name: str = None, ad_group_id_filter: str = None,
                      campaign_id_filter: str = None, creative_type: str = None):
        """
        Gets an array of ad groups associated with the client identifier passed in the authorization header,
        filtered by specified criteria.
        :param start_index: Sets a zero-based offset into the requested set of ad groups. Use in conjunction with
        the count parameter to control pagination of the returned array. default:0
        :param count: Sets the number of ad groups in the returned array. Use in conjunction with the startIndex
        parameter to control pagination. For example, to return the first ten ad groups set startIndex=0 and
        count=10. To return the next ten ad groups, set startIndex=10 and count=10, and so on. default: max
        :param name:The returned array includes only ad groups with the specified name.
        :param ad_group_id_filter:The returned array includes only ad groups with identifiers matching those
        specified in the comma-delimited string.
        :param campaign_id_filter:The returned array includes only ad groups associated with campaign identifiers
        matching those specified in the comma-delimited string.
        :param creative_type:Filter by the type of creative the campaign is associated with. To get ad groups
        associated with non-video campaigns specify 'productCollection'. To get ad groups associated with video
        campaigns, this must be set to 'video'. Returns all ad groups if not specified.
        :return:
        """
        self.uri_path = "/sb/adGroups"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "name": name,
            "adGroupIdFilter": ad_group_id_filter,
            "campaignIdFilter": campaign_id_filter,
            "creativeType": creative_type
        }
        return self.execute()

    def get_ad_group_by_id(self, ad_group_id):
        """
        Gets an ad group specified by identifier.
        :param ad_group_id: The identifier of an existing ad group.
        :return:
        """
        self.uri_path = "/sb/adGroups/{}".format(ad_group_id)
        self.method = "get"
        return self.execute()






from ..client import Client


class LandingPageAsin(Client):

    def get_page_asin(self, page_url):
        self.method = "get"
        self.uri_path = "/pageAsins"
        self.headers["pageUrl"] = page_url
        return self.execute()




import requests
from .conf import domain_dict
import json


class ProClient(object):
    def __init__(self, access_token, region, client_id):
        self.access_token = access_token
        self.headers = {
            "Content-Type": "application/json",
            "Authorization": "Bearer {}".format(self.access_token),
            "Amazon-Advertising-API-ClientId": client_id
        }
        self.domain = domain_dict[region]
        self.method = "get"
        self.data = None
        self.uri_path = None
        self.params = None

    def execute(self):
        url = self.domain + self.uri_path
        if self.method == "delete":
            response = requests.delete(url, headers=self.headers).text
            return response
        if self.data:
            self.data = json.dumps(self.data)
        response = requests.request(self.method, url, params=self.params, headers=self.headers, data=self.data)
        return response

    def execute_download(self, url):
        s = requests.Session()
        self.headers.pop("Content-Type")
        self.headers["Accept-encoding"] = "gzip"
        response = s.get(url, headers=self.headers)
        return response.__dict__


class Client(ProClient):
    def __init__(self, access_token, profile_id, region, client_id):
        self.profile_id = profile_id
        super(Client, self).__init__(access_token, region, client_id)
        self.headers["Amazon-Advertising-API-Scope"] = self.profile_id









from ..client import Client


class BudgetRules(Client):

    def get_budget_rules_history_by_campaign_id(self, campaign_id, page_size: int, start_date: str, end_date: str,
                                          next_token: str = None):
        self.uri_path = "/sd/campaigns/{}/budgetRules/budgetHistory".format(campaign_id)
        self.method = "get"
        self.params = {
            "pageSize": page_size,
            "startDate": start_date,
            "endDate": end_date,
            "nextToken": next_token
        }
        return self.execute()

    def get_budget_rules_by_id(self, budget_rule_id):
        self.uri_path = "/sd/budgetRules/{}".format(budget_rule_id)
        self.method = "get"
        return self.execute()

    def create_budget_rules(self, data):
        self.uri_path = "/sd/budgetRules"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_budget_rules(self, page_size: int, next_token: str = None):
        self.uri_path = "/sd/budgetRules"
        self.method = "get"
        self.params = {
            "pageSize": page_size,
            "nextToken": next_token
        }
        return self.execute()

    def update_budget_rules(self, data):
        self.uri_path = "/sd/budgetRules"
        self.method = "put"
        self.data = data
        return self.execute()

    def get_campaign_budget_rules_by_id(self, budget_rule_id, page_size: int, next_token: str = None):
        self.uri_path = "/sd/budgetRules/{}/campaigns".format(budget_rule_id)
        self.method = "get"
        self.params = {
            "pageSize": page_size,
            "nextToken": next_token
        }
        return self.execute()

    def delete_campaign_budget_rules(self, campaign_id, budget_rule_id):
        self.uri_path = "/sd/campaigns/{}/budgetRules/{}".format(campaign_id, budget_rule_id)
        self.method = "delete"
        return self.execute()

    def associate_campaign_by_id(self, campaign_id, budget_rule_ids: list):
        self.uri_path = "/sd/campaigns/{}/budgetRules".format(campaign_id)
        self.method = "post"
        self.data = {
            "budgetRuleIds": budget_rule_ids
        }
        return self.execute()

    def get_campaign_budget_rules_by_campaign_id(self, campaign_id):
        self.uri_path = "/sd/campaigns/{}/budgetRules".format(campaign_id)
        self.method = "get"
        return self.execute()





from ..client import Client


class Campaigns(Client):

    def create_campaigns(self, data):
        self.uri_path = "/v2/sp/campaigns"
        self.method = "post"
        self.data = data
        return self.execute()

    def update_campaigns(self, data):
        self.uri_path = "/v2/sp/campaigns"
        self.method = "put"
        self.data = data
        return self.execute()

    def get_campaigns(self, start_index: int = 0, count: int = None, state_filter: str = None,
                      name: str = None, portfolio_id_filter: str = None, campaign_id_filter: str = None):
        queries = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "name": name,
            "portfolioIdFilter": portfolio_id_filter,
            "campaignIdFilter": campaign_id_filter,
        }
        self.params = queries
        self.method = "get"
        self.uri_path = "/v2/sp/campaigns"
        return self.execute()

    def get_campaign_by_id(self, campaign_id):
        self.method = "get"
        self.uri_path = "/v2/sp/campaigns/{}".format(campaign_id)
        return self.execute()

    def delete_campaign_by_id(self, campaign_id):
        self.uri_path = "/v2/sp/campaigns/{}".format(campaign_id)
        self.method = "delete"
        return self.execute()

    def get_campaign_extended(self, start_index: int = 0, count: int = None, state_filter: str = None,
                              name: str = None, portfolio_id_filter: str = None, campaign_id_filter: str = None):
        self.method = "get"
        self.uri_path = "/v2/sp/campaigns/extended"
        queries = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "name": name,
            "portfolioIdFilter": portfolio_id_filter,
            "campaignIdFilter": campaign_id_filter,
        }
        self.params = queries
        return self.execute()

    def get_campaign_extended_by_id(self, campaign_id):
        self.method = "get"
        self.uri_path = "/v2/sp/campaigns/{}".format(campaign_id)
        return self.execute()


class CampaignsV3(Client):

    def create_campaigns(self, data):
        self.method = "post"
        self.uri_path = "/sp/campaigns"
        self.data = data
        return self.execute()

    def get_campaigns_list(self, data):
        self.method = "post"
        self.uri_path = "/sp/campaigns/list"
        self.data = data
        return self.execute()

    def update_campaigns(self, data):
        self.method = "put"
        self.uri_path = "/sp/campaigns"
        self.data = data
        return self.execute()

    def delete_campaigns(self, data):
        self.method = "post"
        self.uri_path = "/sp/campaigns/delete"
        self.data = data
        return self.execute()




from setuptools import setup, find_packages


setup(
    name="python-amazon-advertising-api",
    version="0.7.8",
    install_requires=[
        "requests"
    ],
    packages=find_packages(),
    url="https://github.com/854350999/python-amazon-advertising-api",
    license="MIT",
    author="WT",
    author_email="854350999@qq.com",
    description="Python wrapper for the Amazon Advertising API",
    long_description="Python wrapper for the Amazon Advertising API",
    include_package_data=True,
    zip_safe=False,
)






from ..client import Client


class Media(Client):

    def create_media_upload(self, program_type: str = "SponsoredBrands", creative_type: str = "Video"):
        self.uri_path = "/media/upload"
        self.method = "post"
        self.data = {
            "programType": program_type,
            "creativeType": creative_type
        }
        return self.execute()

    def update_media_upload(self, upload_location: str, version: str = None):
        self.uri_path = "/media/complete"
        self.method = "put"
        self.data = {
            "uploadLocation": upload_location,
            "version": version
        }
        return self.execute()

    def get_media_describe(self, media_id):
        self.method = "get"
        self.uri_path = "/media/describe"
        self.headers["mediaId"] = media_id
        return self.execute()





from ..client import Client


class SuggestedKeywords(Client):

    def get_suggested_keywords_by_ad_group_id(self, ad_group_id, max_num_suggestions: int = 100,
                                              ad_state_filter: str = None):
        self.method = "get"
        self.uri_path = "/v2/sp/adGroups/{}/suggested/keywords".format(ad_group_id)
        self.params = {
            "maxNumSuggestions": max_num_suggestions,
            "adStateFilter": ad_state_filter
        }
        return self.execute()

    def get_suggested_keywords_extended_by_ad_group_id(self, ad_group_id, max_num_suggestions: int = 100,
                                                       suggest_bids: str = "no", ad_state_filter: str = None):
        self.method = "get"
        self.uri_path = "/v2/sp/adGroups/{}/suggested/keywords/extended".format(ad_group_id)
        self.params = {
            "maxNumSuggestions": max_num_suggestions,
            "suggestBids": suggest_bids,
            "adStateFilter": ad_state_filter
        }
        return self.execute()

    def get_suggest_keywords_by_asin(self, asin, max_num_suggestions: int = 100):
        self.method = "get"
        self.uri_path = "/v2/sp/asins/{}/suggested/keywords".format(asin)
        self.params = {
            "maxNumSuggestions": max_num_suggestions
        }
        return self.execute()

    def get_suggest_keywords_by_asins(self, asins: list = None, max_num_suggestions: int = 100):
        self.method = "post"
        self.uri_path = "/v2/sp/asins/suggested/keywords"
        self.data = {
            "asins": asins,
            "maxNumSuggestions": max_num_suggestions
        }
        return self.execute()








from ..client import Client


class Brands(Client):

    def get_brands(self, brand_type_filter):
        self.uri_path = "/brands"
        self.method = "get"
        self.params = {
            "brandTypeFilter": brand_type_filter
        }
        return self.execute()




from ..client import Client


class Snapshots(Client):

    def create_snapshots(self, record_type, data):
        self.uri_path = "/sd/{}/snapshot".format(record_type)
        self.method = "post"
        self.data = data
        return self.execute()

    def get_snapshots(self, snapshot_id):
        self.uri_path = "/sd/snapshots/{}".format(snapshot_id)
        self.method = "get"
        return self.execute()

    def get_snapshots_download(self, snapshot_id):
        self.uri_path = "/sd/snapshots/{}/download".format(snapshot_id)
        self.method = "get"
        return self.execute()





from ..client import Client


class Moderation(Client):

    def get_moderation_campaigns_by_id(self, campaign_id):
        self.method = "get"
        self.uri_path = "/sb/moderation/campaigns/{}".format(campaign_id)
        return self.execute()





from ..client import Client


class BidRecommendations(Client):

    def get_bid_recommendations(self, data):
        self.uri_path = "/sd/targets/bid/recommendations"
        self.method = "post"
        self.data = data
        self.headers.update(
            {"Content-Type": "application/vnd.sdtargetingrecommendations.v3.2+json"}
        )
        return self.execute()



from ..client import Client


class AdGroups(Client):

    def get_ad_groups(self, start_index: int = 0, count: int = None, state_filter: str = "enabled, paused, archived",
                      campaign_id_filter: str = None, ad_group_id_filter: str = None, name: str = None):
        self.uri_path = "/sd/adGroups"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "campaignIdFilter": campaign_id_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "name": name
        }
        return self.execute()

    def update_ad_groups(self, data):
        self.uri_path = "/sd/adGroups"
        self.method = "put"
        self.data = data
        return self.execute()

    def create_ad_groups(self, data):
        self.uri_path = "/sd/adGroups"
        self.method = "post"
        self.data = data
        return self.execute()

    def get_ad_group_by_id(self, ad_group_id):
        self.uri_path = "/sd/adGroups/{}".format(ad_group_id)
        self.method = "get"
        return self.execute()

    def delete_ad_group_by_id(self, ad_group_id):
        self.uri_path = "/sd/adGroups/{}".format(ad_group_id)
        self.method = "delete"
        return self.execute()

    def get_ad_group_extended(self, start_index: int = 0, count: int = None,
                              state_filter: str = "enabled, paused, archived",
                              campaign_id_filter: str = None, ad_group_id_filter: str = None, name: str = None):
        self.uri_path = "/sd/adGroups/extended"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "campaignIdFilter": campaign_id_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "name": name
        }
        return self.execute()

    def get_ad_group_extended_by_id(self, ad_group_id):
        self.uri_path = "/sd/adGroups/extended/{}".format(ad_group_id)
        self.method = "get"
        return self.execute()



from ..client import Client


class ProductTargeting(Client):

    def create_targets(self, params):
        self.uri_path = "/v2/sp/targets"
        self.method = "post"
        self.data = params
        return self.execute()

    def update_targets(self, params):
        self.uri_path = "/v2/sp/targets"
        self.method = "put"
        self.data = params
        return self.execute()

    def get_targets(self, start_index: int = 0, count: int = None, state_filter: str = None,
                    campaign_id_filter: str = None, ad_group_id_filter: str = None, target_id_filter: str = None):
        self.uri_path = "/v2/sp/targets"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "campaignIdFilter": campaign_id_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "targetIdFilter": target_id_filter
        }
        return self.execute()

    def get_target_by_id(self, target_id):
        self.method = "get"
        self.uri_path = "/v2/sp/targets/{}".format(target_id)
        return self.execute()

    def delete_target_by_id(self, target_id):
        self.uri_path = "/v2/sp/targets/{}".format(target_id)
        self.method = "delete"
        return self.execute()

    def get_targets_extended(self, start_index: int = 0, count: int = None, state_filter: str = None,
                             campaign_id_filter: str = None, ad_group_id_filter: str = None,
                             target_id_filter: str = None):
        self.uri_path = "/v2/sp/targets/extended"
        self.method = "get"
        self.params = {
            "startIndex": start_index,
            "count": count,
            "stateFilter": state_filter,
            "campaignIdFilter": campaign_id_filter,
            "adGroupIdFilter": ad_group_id_filter,
            "targetIdFilter": target_id_filter
        }
        return self.execute()

    def get_targets_extended_by_id(self, target_id):
        self.method = "get"
        self.uri_path = "/v2/sp/targets/extended/{}".format(target_id)
        return self.execute()

    def get_product_recommendations_targets(self, params):
        self.uri_path = "/v2/sp/targets/productRecommendations"
        self.data = params
        self.method = "post"
        return self.execute()

    def get_list_categories_targets(self, asins):
        self.method = "get"
        self.uri_path = "/v2/sp/targets/categories"
        self.data = {
            "asins": asins
        }
        return self.execute()

    def get_categories_refinements_targets(self, category_id):
        self.method = "get"
        self.uri_path = "/v2/sp/targets/categories/refinements"
        self.data = {
            "categoryId": category_id
        }
        return self.execute()

    def get_brands_targets(self, params):
        self.method = "get"
        self.uri_path = "/v2/sp/targets/brands"
        self.data = params
        return self.execute()


class ProductTargetingV3(Client):

    def get_targets_list(self, data):
        self.method = "post"
        self.uri_path = "/sp/targets/list"
        self.data = data
        return self.execute()

    def update_targets(self, data):
        self.method = "put"
        self.uri_path = "/sp/targets"
        self.data = data
        return self.execute()

    def delete_targets(self, data):
        self.method = "post"
        self.uri_path = "/sp/targets/delete"
        self.data = data
        return self.execute()
